/*
 *     SimpleBrowser.java  Jan 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.gui;
import org.jtool.platform.j2xml.SystemProperties;
import org.jtool.platform.j2xml.Repository;
import javax.swing.*;
import java.util.Locale;
import java.io.*;

public class SimpleBrowser {
    protected String applicationName;
    protected MainFrame frame;
    
    public static void main(String[] args) {
        SimpleBrowser browser = new SimpleBrowser();
        browser.start();
    }

    public SimpleBrowser() {
        this("Java Code Browser");
    }

    public SimpleBrowser(String appName) {
        setLookAndFeel();

        applicationName = appName;
        SystemProperties.setApplicationDir(".");

        loadProperies();        
        setProjectDirProperty();
    }

    public void start() {
        frame = createMainFrame();
        frame.createPanes();
        frame.initPanes();

        frame.pack();
        frame.validate();
        frame.setVisible(true);
    }

    protected MainFrame createMainFrame() {
        return new MainFrame(this, applicationName);
    }

    private void setLookAndFeel() {
        /*
        if (System.getProperty("os.name").startsWith("Windows")) {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            } catch(Exception e) { }
        }
        */
        Locale.setDefault(Locale.ENGLISH);
    }

    private void loadProperies() {
        if (!SystemProperties.loadSystemProperties()) {
            JOptionPane.showMessageDialog(null, "Fail to load properties from the configuration file.");
            System.exit(1);
        }

        File file = new File(SystemProperties.getSapidDir());
        if (!file.isDirectory()) {
            JOptionPane.showMessageDialog(null, "Check Sapid directory.");
            System.exit(1);
        }
    }

    private void setProjectDirProperty() {
        while (!checkProjectDir()) {
            ProjectPathDialog dialog = new ProjectPathDialog(this, "Project Path");
            dialog.show();
            dialog.dispose();
        }

        setProjectDir(getProjectDir());
    }

    private boolean checkProjectDir() {
        String projectDir = SystemProperties.getProjectDir();

        if (projectDir != null) {
            File file = new File(projectDir);
            if (file != null && file.isDirectory()) {
                return true;
            }
        }
        return false;
    }

    public void setProjectDir(String dirName) {
        SystemProperties.setProjectDir(dirName);
        SystemProperties.setClassPath(dirName);
        SystemProperties.setSourceDir(dirName);

        SystemProperties.loadProjectProperties();
    }

    public String getProjectDir() {
        return SystemProperties.getProjectDir();
    }

    public void setClassPath(String path) {
        SystemProperties.setClassPath(path);
    }

    public String getClassPath() {
        return SystemProperties.getClassPath();
    }

    public void terminate() {
        SystemProperties.storeSystemProperties();
        Repository.terminate();
        System.exit(0);
    }
}
