/*
 *     AncestorResolver.java  Nov 21, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.util.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class AncestorResolver extends SimpleResolver {

    public static void canUseAncestors(boolean bool) {
        GetFqn.canUseAncestors(bool);
    }

    public static JavaClassList getAncestors(JavaClass jc) {
        String pathName = jc.getJavaFile().getName();
        if (pathName != null) {
            XMLText text = Repository.getXMLText(pathName);

            if (text != null) {
                if (!text.hasSetAncestors()) {
                    setAncestors(text);
                }
                return text.getAncestors(jc);
            }
        }
        return new JavaClassList();
    }

    public static void setAncestors(ArrayList files) {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            XMLText text = (XMLText)it.next();
            setAncestors(text);
        }
    }

    public static void setAncestors(XMLText text) {
        if (!text.hasSetAncestors()) {
            JavaFile jf = new JavaFile(text.getDocument().getDocumentElement());
            setAncestors(text, jf.getClassNodes());
            setAncestors(text, jf.getInterfaceNodes());
        }
    }

    private static void setAncestors(XMLText text, ArrayList nodes) {
        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            setAncestors(text, jc);
        }
    }

    public static void setAncestors(XMLText text, JavaClass jc) {
        JavaClassList classes = new JavaClassList();
        setAncestors(jc, classes, 0);

        text.setAncestors(jc, classes);
        text.setResolvedFqn(classes);
        writeAncestors(text, jc.getElem(), classes);
    }

    private static void setAncestors(JavaClass jclass, JavaClassList classes, int d) {
        if (jclass != null) {
            if (classes.getJavaClass(jclass.getFqn()) == null) {
                classes.add(new AncestorOrDescendant(jclass, d));
            }

            if (!jclass.getFqn().equals("java.lang.Object")) {
                String className = jclass.getSuperClassName();
                if (className != null) {
                    JavaClass jc = Repository.entryClass(className);
                    setAncestors(jc, classes, d + 1);
                }

                Iterator it = jclass.getSuperInterfaceNames().iterator();
                while (it.hasNext()) {
                    className = (String)it.next();
                    JavaClass jc = Repository.entryClass(className);
                    setAncestors(jc, classes, d + 1);
                }
            }
        }
    }

    public static JavaClassList getDescendants(JavaClass jc) {
        String pathName = jc.getJavaFile().getName();
        if (pathName != null) {
            XMLText text = Repository.getXMLText(pathName);;
            if (text != null) {
                if (!text.hasSetDescendants()) {
                    setDescendants(text);
                }
                return text.getDescendants(jc);
            }
        }
        return new JavaClassList();
    }

    public static void setDescendants(ArrayList files) {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            AncestorResolver.setDescendants(text);
        }
    }

    public static void setDescendants(XMLText text) {
        JavaFile jf = new JavaFile(text.getDocument().getDocumentElement());
        setDescendants(text, jf.getClassNodes());
        setDescendants(text, jf.getInterfaceNodes());
    }

    private static void setDescendants(XMLText text, ArrayList nodes) {
        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            setDescendants(text, jc);
        }
    }

    private static void setDescendants(XMLText text, JavaClass jc) {
        JavaClassList classes = new JavaClassList();
        setDescendants(jc, classes);

        text.setDescendants(jc, classes);
        text.setResolvedFqn(classes);
        writeDescendants(text, jc.getElem(), classes);
    }

    private static void setDescendants(JavaClass jclass, JavaClassList classes) {
        Iterator it = Repository.getJavaTexts().iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            JavaFile jf = new JavaFile(text.getDocument().getDocumentElement());

            Iterator itc = jf.getClassAndInterfaceNodes().iterator();
            while (itc.hasNext()) {
                JavaClass jc = new JavaClass((Node)itc.next());

                JavaClassList ancestors = AncestorResolver.getAncestors(jc);
                AncestorOrDescendant ad = (AncestorOrDescendant)ancestors.getJavaClass(jclass);
                if (ad != null) {  // ancestors.contains(jclass)
                    classes.add(new AncestorOrDescendant(jc, ad.getDistance()));
                }
            }
        }
    }

    public static void writeAncestors(XMLText text, Element elem, JavaClassList classes) {
        writeAncestorsOrDescendants(text, elem, classes, JXConstants.AncestorElem);
    }

    public static void writeDescendants(XMLText text, Element elem, JavaClassList classes) {
        writeAncestorsOrDescendants(text, elem, classes, JXConstants.DescendantElem);
    }

    private static void writeAncestorsOrDescendants(XMLText text, Element elem,
                                                    JavaClassList classes, String elemName) {
        Document doc = text.getDocument();
        removeChildNodes(elem, elemName);

        Iterator it = classes.iterator();
        while (it.hasNext()) {
            AncestorOrDescendant ad = (AncestorOrDescendant)it.next();

            Element e = (Element)appendElemChildNode(doc, elem, elemName);
            e.setAttribute(JXConstants.NameAttr, ad.getFqn());
            e.setAttribute(JXConstants.DistanceAttr, String.valueOf(ad.getDistance()));
            if (ad.isInterface()) {
                e.setAttribute(JXConstants.SortAttr, JXConstants.InterfaceValue);
            } else {
                e.setAttribute(JXConstants.SortAttr, JXConstants.ClassValue);
            }
        }
    }
}

class AncestorOrDescendant extends JavaClass {
    private int distance;
    
    AncestorOrDescendant(JavaClass jc, int dist) {
        super(jc.getElem());
        distance = dist;
    }

    int getDistance() {
        return distance;
    }
}
