/*
 *     ClassFileParser.java  Dec 3, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml;
import org.apache.bcel.classfile.*;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.event.*;
import java.io.*;
import java.util.*;
import java.util.jar.*;
import java.util.zip.*;

public class ClassFileParser {
    private static JavaClass jclass;

    public static JavaClass parse(String pathName) {
        String jar = ClassPathFile.getJarFileName(pathName);
        return parse(pathName, jar);
    }

    public static JavaClass parse(String pathName, String jar) {
        if (pathName != null) {
            try {
                ClassParser parser;
                if (jar != null) {
                    pathName = pathName.replace(File.separatorChar, '/');
                    parser = new ClassParser(jar, pathName);
                } else {
                    parser = new ClassParser(pathName);
                }
                jclass = parser.parse();
                
                // EventManager.getInstance().fire(
                //   new NoticeEvent(new Repository(), "PARSE: " + pathName));
                
            } catch (IOException e) {
                // System.err.println("BCEL: Parse error occurred.");
                jclass = null;
            }
            return jclass;
        }
        return null;
    }

    public static String createXMLText(String pathName) {
        JavaClass jc = parse(pathName);
        if (jc != null) {
            return writeXMLText(pathName);
        }
        return null;
    }

    private static StringBuffer buf;
    private static int id;

    private static String writeXMLText(String pathName) {
        buf = new StringBuffer();
        if (jclass != null) {
            id = 0;
            writeFileElem(jclass, pathName);
        }
        return new String(buf.toString());
    }

    private static void writeFileElem(JavaClass jclass, String pathName) {
        buf.append("<" + JXConstants.FileElem + getIdAttr(getIncId()));
        buf.append(" " + JXConstants.NameAttr + "=\"" + jclass.getClassName() + "\"");
        buf.append(" " + JXConstants.PathAttr + "=\"" + pathName + "\"");
        String jar = ClassPathFile.getJarFileName(pathName);
        if (jar != null) {
            buf.append(" " + JXConstants.JarAttr + "=\"" + jar + "\"");
        }
        buf.append(" " + JXConstants.SortAttr + "=\"" + JXConstants.LEVELC + "\"");
        buf.append(">\n");
        writePackageElem(jclass);
        writeClassElem(jclass);
        buf.append("</" + JXConstants.FileElem + ">\n");
    }

    private static void writePackageElem(JavaClass jclass) {
        String packageName = jclass.getPackageName();
        if (packageName.length() != 0) { 
            buf.append("<" + JXConstants.PackageElem + getIdAttr(getIncId()) + ">");
            writeQNameElem(jclass.getPackageName());
            buf.append("</" + JXConstants.PackageElem + ">\n");
        }
    }

    private static void writeClassElem(JavaClass jclass) {
        String elem = JXConstants.ClassElem;
        if (jclass.isInterface()) {
            elem = JXConstants.InterfaceElem;
        }
        buf.append("<" + elem + getIdAttr(getIncId()));
        buf.append(getAccessAttr(jclass));
        buf.append(getFinalAttr(jclass));
        buf.append(getAbstractAttr(jclass));
        buf.append(getStaticAttr(jclass));
        buf.append(getFqnAttr(jclass.getClassName()));
        buf.append(">");

        String className = jclass.getClassName();
        int sep = jclass.getClassName().lastIndexOf(".");
        if (sep != -1) {
            className = className.substring(sep + 1);
        }
        writeIdentElem(className);
        buf.append("\n");

        if (!jclass.isInterface()) {
            writeExtendsElem(jclass);
        }
        writeImplementsElem(jclass);
        writeMembersElem(jclass);
        buf.append("</" + elem + ">\n");
    }

    private static void writeExtendsElem(JavaClass jclass) {
        buf.append("<" + JXConstants.ExtendsElem + getIdAttr(getIncId()));
        buf.append(getTypeFirstAttr(getIncId()) + ">");
        writeTypeElem(jclass.getSuperclassName(), getIdAttr(getId()));
        buf.append("</" + JXConstants.ExtendsElem + ">\n");
    }

    private static void writeImplementsElem(JavaClass jclass) {
        String impls[] = jclass.getInterfaceNames();
        for (int i = 0; i < impls.length; i++) {
            buf.append("<" + JXConstants.ImplementsElem + getIdAttr(getIncId()));
            buf.append(getTypeFirstAttr(getIncId()) + ">");
            writeTypeElem(impls[i], getIdAttr(getId()));
            buf.append("</" + JXConstants.ImplementsElem + ">\n");
        }
    }

    private static void writeMembersElem(JavaClass jclass) {
        buf.append("<" + JXConstants.MembersElem + ">\n");
        writeFieldElem(jclass);
        writeMethodElem(jclass);
        buf.append("</" + JXConstants.MembersElem + ">\n");
    }

    private static void writeFieldElem(JavaClass jclass) {
        Field[] fields = jclass.getFields();
        for (int i = 0; i < fields.length; i++) {
            buf.append("<" + JXConstants.FieldElem + getIdAttr(getIncId()));
            buf.append(getAccessAttr(fields[i]));
            buf.append(getFinalAttr(fields[i]));
            buf.append(getAbstractAttr(fields[i]));
            buf.append(getStaticAttr(fields[i]));
            buf.append(getSynchronizedAttr(fields[i]));
            buf.append(getNativeAttr(fields[i]));
            buf.append(getVolatileAttr(fields[i]));
            buf.append(getTransientAttr(fields[i]));
            buf.append(getTypeFirstAttr(getIncId()) + ">");
            writeTypeElem(getFieldType(fields[i]), getIdAttr(getId()));
            writeIdentElem(fields[i].getName());
            buf.append("</" + JXConstants.FieldElem + ">\n");
        }
    }

    private static void writeMethodElem(JavaClass jclass) {
        Method[] methods = jclass.getMethods();
        for (int i = 0; i < methods.length; i++) {
            String name = methods[i].getName();
            String elem = JXConstants.MethodElem;

            if (!name.equals("<clinit>")) {  // not static-initializer

                if (name.equals("<init>")) {
                    elem = JXConstants.ConstructorElem;
                    name = jclass.getClassName();
                    int index = name.lastIndexOf(".");
                    if (index != -1) {
                        name = name.substring(index + 1);
                    }
                }
                buf.append("<" + elem + getIdAttr(getIncId()));
                buf.append(getAccessAttr(methods[i]));
                buf.append(getFinalAttr(methods[i]));
                buf.append(getAbstractAttr(methods[i]));
                buf.append(getStaticAttr(methods[i]));
                buf.append(getSynchronizedAttr(methods[i]));
                buf.append(getNativeAttr(methods[i]));
                // buf.append(getTransientAttr(methods[i]));
                // buf.append(getStrictfpAttr(methods[i]));

                if (elem == JXConstants.MethodElem) {
                    buf.append(getTypeFirstAttr(getIncId()) + ">");
                    writeTypeElem(getMethodType(methods[i]), getIdAttr(getId()));
                } else {
                    buf.append(">");
                }
                writeIdentElem(name);
                buf.append("\n");
                writeParameterElem(methods[i]);
                buf.append("</" + elem + ">\n");
            }
        }
    }

    private static void writeParameterElem(Method method) {
        String[] type = getParameters(method);
        for (int i = 0; i < type.length; i++) {
            buf.append("<" + JXConstants.ParameterElem + getIdAttr(getIncId()));
            buf.append(getTypeFirstAttr(getIncId()) + ">");
            writeTypeElem(type[i], getIdAttr(getId()));
            buf.append("</" + JXConstants.ParameterElem + ">\n");
        }
    }

    private static void writeThrowsElem(String name, String sort) {
        buf.append("<" + JXConstants.ThrowsElem);
        buf.append(">\n");
    }

    private static void writeTypeElem(String type, String idattr) {
        buf.append("<" + JXConstants.TypeElem + idattr + getSortAttr(type));
        buf.append(getFqnAttr(type));
        buf.append(">");
        buf.append("</" + JXConstants.TypeElem + ">");
    }

    private static void writeQNameElem(String name) {
        buf.append("<" + JXConstants.QNameElem + getIdAttr(getIncId()) + ">");
        buf.append("<" + JXConstants.IdentElem + ">");
        buf.append(name);
        buf.append("</" + JXConstants.IdentElem + ">");
        buf.append("</" + JXConstants.QNameElem + ">");
    }

    private static void writeIdentElem(String name) {
        buf.append("<" + JXConstants.IdentElem + ">");
        buf.append(name);
        buf.append("</" + JXConstants.IdentElem + ">");
    }

    private static int getId() {
        return id;
    }

    private static int getIncId() {
        if (id < Integer.MAX_VALUE) {
            id++;
        } else {
            id = 0;
        }
        return id;
    }

    private static String getIdAttr(int i) {
        return " " + JXConstants.IdAttr + "=\"c" + i + "\"";
    }

    private static String getTypeFirstAttr(int i) {
        return " " + JXConstants.TypeFirstAttr + "=\"c" + i + "\"";
    }

    private static String getAccessAttr(AccessFlags flags) {
        String access = null;
        if (flags.isPublic()) {
            access = "Public";
        } else if (flags.isProtected()) {
            access = "Protected";
        } else if (flags.isPrivate()) {
            access = "Private";
        }
        if (access != null) {
            return " " +  JXConstants.AccessAttr + "=\"" + access + "\"";
        }
        return "";
    }

    private static String getFinalAttr(AccessFlags flags) {
        if (flags.isFinal()) {
            return " " + JXConstants.FinalAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getAbstractAttr(AccessFlags flags) {
        if (flags.isAbstract()) {
            return " " + JXConstants.AbstractAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getStaticAttr(AccessFlags flags) {
        if (flags.isStatic()) {
            return " " + JXConstants.StaticAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getSynchronizedAttr(AccessFlags flags) {
        if (flags.isSynchronized()) {
            return " " + JXConstants.SynchronizedAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getNativeAttr(AccessFlags flags) {
        if (flags.isNative()) {
            return " " + JXConstants.NativeAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getTransientAttr(AccessFlags flags) {
        if (flags.isTransient()) {
            return " " + JXConstants.TransientAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getVolatileAttr(AccessFlags flags) {
        if (flags.isVolatile()) {
            return " " + JXConstants.VolatileAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getStrictfpAttr(AccessFlags flags) {
        if (flags.isStrictfp()) {
            return " " + JXConstants.StrictfpAttr + "=\"yes\"";
        }
        return "";
    }

    private static String getSortAttr(String type) {
        String sort = JXConstants.ObjectRef;
        if (TypeConversion.isPrimitiveType(type)) {
            sort = JXConstants.Primitive;
        }
        return " " + JXConstants.SortAttr + "=\"" + sort + "\"";
    }

    private static String getFqnAttr(String name) {
        return " " + JXConstants.FqnAttr + "=\"" + name + "\"";
    }

    private static String getFieldType(Field f) {
        return Utility.signatureToString(f.getSignature(), false);
    }

    private static String getMethodType(Method m) {
        return Utility.methodSignatureReturnType(m.getSignature(), false);
    }

    private static String[] getParameters(Method m) {
        return Utility.methodSignatureArgumentTypes(m.getSignature(), false);
    }

    public static void main(String[] args) {
        if (args.length == 1) {
            System.out.print(ClassFileParser.createXMLText(args[0]));

        } else {
            System.err.println("Usage: java ClassFileParser classfile");
        }
    }
}
