/*
 *     FileResolver.java  Nov 27, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.dom.*;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.util.*;
import org.w3c.dom.*;
import java.util.*;
import java.io.*;

public class FileResolver extends SimpleResolver {
    private static Document document;
    private static JavaElement jelem;
    private static int anonymousNum;

    public static Document run(JavaXMLText text) {
        return run(text.getName(), text.getDocument());
    }

    public static Document run(String fileName, Document doc) {
        document = doc;
        Element elem = document.getDocumentElement();
        jelem = new JavaElement(elem);

        foldQName(elem, JXConstants.PackageElem);
        foldQName(elem, JXConstants.ImportElem);

        elem.setAttribute(JXConstants.PathAttr, fileName);
        elem.setAttribute(JXConstants.SortAttr, JXConstants.LEVEL1);

        JavaFile jfile = new JavaFile(elem);
        String packageName = jfile.getPackageName();

        anonymousNum = 1;

        traverseNode(elem, packageName);
        return document;
    }

    private static void traverseNode(Node node, String packageName) {
        if (node.getNodeType() == Node.ELEMENT_NODE) {
            if (node.getNodeName().equals(JXConstants.ClassElem) ||
                node.getNodeName().equals(JXConstants.InterfaceElem)) {
                writeClassFqn(node, packageName);
            }
        }

        NodeList nl = node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node child = nl.item(i);
            traverseNode(child, packageName);
        }
    }

    private static void foldQName(Element elem, String elemName) {
        NodeList nl = elem.getElementsByTagName(elemName);
        for (int i = 0; i < nl.getLength(); i++) {
            Node node = nl.item(i);
            foldQName(node);
        }
    }

    public static void foldQName(Node node) {
        if (node.getNodeName().equals(JXConstants.QNameElem)) {
            removeNestedQName(node);
        }
        NodeList nl = node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node child = nl.item(i);
            foldQName(child);
        }        
    }

    private static void removeNestedQName(Node node) {
        String qname = collectQName(node);
        removeChildNodes(node);

        Element elem = (Element)appendElemChildNode(document, node, JXConstants.IdentElem);
        String defid = ((Element)node).getAttribute(JXConstants.IdAttr);
        elem.setAttribute(JXConstants.DefIdAttr, defid);

        appendTextChildNode(document, elem, qname);
    }

    public static String collectQName(Node node) {
        StringBuffer buf = new StringBuffer();
        collectQName(node, buf);
        return buf.toString();
    }

    private static void collectQName(Node node, StringBuffer buf) {
        NodeList nl = node.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node child = nl.item(i);
            if (child.getNodeType() == Node.TEXT_NODE) {
                buf.append(child.getNodeValue());
            } else if (child.getNodeType() == Node.ELEMENT_NODE) {
                collectQName(child, buf);
            }
        }
    }

    private static void writeClassFqn(Node node, String packageName) {
        JavaClass jclass = new JavaClass(node);

        String name;
        if (jclass.isInnerClass()) {
            Element e = (Element)jclass.getOuterClassNode();
            JavaClass jc = new JavaClass(e);

            if (jclass.isAnonymous()) {
                name = jc.getFqn() + "$" + String.valueOf(anonymousNum);
                anonymousNum++;
            } else {
                name = jc.getFqn() + "$" + jclass.getName();
            }

        } else {
            name = jclass.getName();
            if (packageName.length() != 0) {
                name = packageName + "." + name;
            }
        }
        ((Element)node).setAttribute(JXConstants.FqnAttr, name);
        rewriteType(jclass);
    }

    private static void rewriteType(JavaClass jclass) {
        Iterator it = jclass.getDescendantNodes(jclass.getElem(), JXConstants.TypeElem).iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            String id = elem.getAttribute(JXConstants.QNameAttr);
            if (id.length() != 0) {
                Element qelem = jclass.getQNameNode(elem, id);

                foldQName(qelem);

                String ident = jclass.getIdentName(qelem) + "." + jclass.getIdentName(elem);

                elem.removeAttribute(JXConstants.QNameAttr);
                removeChildNodes(qelem);
                Node parent = qelem.getParentNode();
                parent.removeChild(qelem.getNextSibling());
                parent.removeChild(qelem);

                Node node = jclass.getFirstChildNode(elem, JXConstants.IdentElem);
                node.getFirstChild().setNodeValue(ident);
            }
        }
    }

    private static void writeType(JavaClass jclass, Element elem, String ident) {
        Node node = jclass.getFirstChildNode(elem, JXConstants.IdentElem).getFirstChild();
        node.setNodeValue(ident);
    }
}
