/*
 *     GetFqn.java  Jan 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.j2xml;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.dom.*;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.util.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class GetFqn {
    private static ArrayList candidates = new ArrayList();  // String
    private static ArrayList fileList = new ArrayList();  // String
    private static HashMap fileMap = new HashMap();  // String/String
    private static HashMap fqnCache = new HashMap();  // String/String
    private static boolean canUseAncestors = false;

    public static void canUseAncestors(boolean bool) {
        canUseAncestors = bool;
    }
    
    public static void init(JavaFile jfile) {
        candidates.clear();
        fileList.clear();
        fileMap.clear();
        fqnCache.clear();

        String fileName = jfile.getName();
        String packageName = jfile.getPackageName();
        ArrayList importNames = new ArrayList();

        Iterator it = jfile.getImportNames().iterator();
        while (it.hasNext()) {
            String importName = (String)it.next();
            if (importName.indexOf("*") == -1) {
                importNames.add(0, importName);
            } else {
                importNames.add(importName);
            }
        }

        candidates.addAll(getFilesUnderProject(Repository.getJavaTexts()));
        if (packageName.length() == 0) {  // anonymous package
            candidates.addAll(getFilesInDir(jfile.getDirName()));
        } else {
            candidates.addAll(ClassPathFile.collectCandidates(packageName));
        }
        candidates.addAll(ClassPathFile.collectCandidates(importNames));

        collectAccessibleFiles(candidates, fileName, packageName);
    }

    private static void printCandidates(ArrayList files) {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            String name = (String)it.next();
            System.out.println("CANDIDATE = " + name);
        }
    }

    private static ArrayList getFilesUnderProject(ArrayList texts) {
        ArrayList files = new ArrayList();

        Iterator it = texts.iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            files.add(text.getName());
        }
        return files;
    }

    private static ArrayList getFilesInDir(String dirName) {
        ArrayList files = new ArrayList();

        File dir = new File(dirName);
        if (dir.isDirectory()) {
            String[] names = dir.list();
            for (int i = 0; i < names.length; i++) {
                String name = dirName + File.separator + names[i];

                File file = new File(name);
                if (file.isFile() &&
                    (name.endsWith(".java") || name.endsWith(".class"))) {
                    files.add(file.getPath());
                }
            }
        }
        return files;
    }

    private static void collectAccessibleFiles(ArrayList files,
                                               String fileName, String packegeName) {
        Iterator it = files.iterator();
        while (it.hasNext()) {
            String name = (String)it.next();
            addAccessibleFile(name, fileName, packegeName);
        }
    }

    private static void addAccessibleFile(String name, String fileName, String packageName) {
        if (name.endsWith(".java")) {
            ArrayList files = ClassPathFile.getJavaClassFiles(name);
            if (files != null) {
                Iterator it = files.iterator();

                while (it.hasNext()) {
                    ClassFileName cname = (ClassFileName)it.next();
                    if (fileName.equals(name) || cname.isPublic() ||
                        packageName.equals(cname.getPackageName())) {
                        String className = cname.getName();

                        if (getFileName(className) == null) {
                            fileList.add(className);
                            fileMap.put(className, name);
                        }
                    }
                }
            }

        } else if (name.endsWith(".class")) {
            ClassFileName cname = ClassPathFile.getClassFile(name);
            if (cname != null && (cname.isPublic() ||
                 (packageName != null && packageName.equals(cname.getPackageName())))) {
                String className = cname.getName();

                if (getFileName(className) == null) {
                    fileList.add(className);
                    fileMap.put(className, name);
                }
            }
        }
    }

    public static String getClassFileName(String fqn) {
        String name = getFileName(fqn);
        if (name != null) {
            return name;
        }
        return null;
    }        
    
    private static String getFileName(String fqn) {
        if (fqn != null) {
            int sep = fqn.indexOf("[");
            if (sep != -1) {
                fqn = fqn.substring(0, sep);
            }
            return (String)fileMap.get(fqn);
        }
        return null;
    }

    public static String getFqn(JavaClass jclass, String name) {
        if (name != null) {
            String array = "";
            int sep = name.indexOf("[");
            if (sep != -1) {
                array = name.substring(sep);
                name = name.substring(0, sep);
            }
            
            if (name.length() == 0 ||
                name.equals("null") || TypeConversion.isPrimitiveType(name)) {
                return new String(name + array);
            }

            String qname = (String)fqnCache.get(name);
            if (qname != null) {
                return qname + array;
            }

            if (name.indexOf(".") != -1) {
                String pathName = ClassPathFile.getClassFileName(name);
                if (pathName != null) {
                    fqnCache.put(name, name);
                    return new String(name + array);
                }
            }

            qname = getMatchedName(name, jclass);
            if (qname != null) {
                fqnCache.put(name, qname);
                return new String(qname + array);
            }

            while (name.indexOf(".") != -1) {
                sep = name.lastIndexOf(".");
                name = name.substring(0, sep) + "$" + name.substring(sep + 1);

                qname = getMatchedName(name, jclass);
                if (qname != null) {
                    fqnCache.put(name, qname);
                    return new String(qname + array);
                }
            }
        }
        return null;
    }

    private static String getMatchedName(String className, JavaClass jclass) {
        ArrayList names = getCandidateClassNames(jclass);

        Iterator it = fileList.iterator();
        while (it.hasNext()) {
            String fqn = (String)it.next();
            fqn = fqn.replace(File.separatorChar, '.');

            Iterator itn = names.iterator();
            while (itn.hasNext()) {
                String name = (String)itn.next();
                // System.out.println("COMPARE = " + fqn + " " + name + "-" + className);

                if (fqn.startsWith(name)) {
                    String cname = "." + fqn;
                    cname = "." + cname.replace('$', '.');
                    if (cname.endsWith("." + className)) {
                        return fqn;
                    }
                }
            }
        }
        
        it = fileList.iterator();
        while (it.hasNext()) {
            String fqn = (String)it.next();
            fqn = fqn.replace(File.separatorChar, '.');
            // System.out.println("COMPARE = " + fqn + " " + className);
            
            String cname = "." + fqn;
            if (cname.endsWith("." + className)) {
                return fqn;
            }
        }

        return null;
    }

    private static ArrayList getCandidateClassNames(JavaClass jclass) {
        String packageName = jclass.getJavaFile().getPackageName();

        ArrayList names = new ArrayList();

        JavaClassList classes = new JavaClassList();
        classes.add(jclass);
        classes.addAll(jclass.getAllOuterClasses());

        Iterator it1 = classes.iterator();
        while (it1.hasNext()) {
            JavaClass jc1 = (JavaClass)it1.next();
            names.add(jc1.getFqn());

            if (canUseAncestors) {
                Iterator it2 = AncestorResolver.getAncestors(jc1).iterator();
                while (it2.hasNext()) {
                    JavaClass jc2 = (JavaClass)it2.next();

                    if (jc2.getJavaFile().getPackageName().equals(packageName)) { 
                        names.add(jc2.getFqn());
                    }
                }
            }
        }
        return names;
    }        
}
