/*
 *     MillxWindows.java  Nov 21, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.millx;
import org.jtool.platform.j2xml.*;
import org.jtool.platform.util.*;
import java.io.*;

public class MillxWindows extends Millx {
    private static String bash;
    private static String cygwinDir;
    private static String millxCommand;
    private static String options;

    public static void init(String opt) {
        cygwinDir = SystemProperties.getCygwinDir();

        if (sapidDir.startsWith(cygwinDir)) {
            sapidDir = sapidDir.substring(cygwinDir.length());
        } else {
            System.err.println("Error: Sapid directory: " + sapidDir);
            return;
        }
        sapidDir = sapidDir.replace(File.separatorChar, '/');

        millxCommand = sapidDir + "/bin/millx";
        bash = cygwinDir + "\\bin\\bash";
        options = opt;
    }

    public static void run(String fileName, String xmlDir) throws Exception {
        try {
            String[] cmd = new String[3];
            cmd[0] = bash;
            cmd[1] = "-c";
            if (xmlDir != null) {
                cmd[2] = "\"" + millxCommand + " -sdbD " + xmlDir + " " + options +
                    " '" + fileName + "'" + "\"";
            } else {
                cmd[2] = "\"" + millxCommand + " " + options +
                    " '" + fileName + "'" + "\"";
            }
            
            // System.out.println(cmd[0] + " " + cmd[1] + " " + cmd[2]);

            Runtime rt = Runtime.getRuntime();
            Process process = rt.exec(cmd);
            consumeStream(process);
            // System.out.println(getMessage(process.getErrorStream()));

        } catch (Exception e) {
            throw e;
        }
    }

    private static void consumeStream(Process process) throws Exception {
        DataInputStream odis = new DataInputStream(process.getInputStream());
        BufferedReader obr = new BufferedReader(new InputStreamReader(odis));

        DataInputStream edis = new DataInputStream(process.getErrorStream());
        BufferedReader ebr = new BufferedReader(new InputStreamReader(edis));

        String line;
        while (true) {
            if(odis.available() > 0 ) {
                while((line = obr.readLine()) != null) {
                    // System.out.println(line);
                }
                break;

            } else if(edis.available() > 0 ) {
                while((line = ebr.readLine()) != null) {
                    // System.out.println(line);
                }
                break;

            } else{
                Thread.sleep(200);
            }

            try {
                process.exitValue();
                break;
                
            } catch (IllegalThreadStateException e) {
                continue;
            }            
        }
    }
}
