/*
 *     GetSortAtPosition.java  Dec 19, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.dom;
import org.jtool.platform.xml.JXConstants;
import java.util.*;
import org.w3c.dom.*;

public class GetSortAtPosition {
    private Document document;
    private int startPos, endPos;
    private ArrayList foundSorts = new ArrayList();  // String
    private ArrayList foundDefIds = new ArrayList();  // String
    private ArrayList foundElems = new ArrayList();  // Element

    public GetSortAtPosition(Document doc, int start) {
        this(doc, start, start);
    }

    public GetSortAtPosition(Document doc, int start, int end) {
        document = doc;
        startPos = start;
        endPos = end;
    }

    public String run() {
        try {
            checkNode(document.getDocumentElement());

        } catch (Exception e) {
            foundSorts.clear();
            foundDefIds.clear();
            foundElems.clear();
        }
        return getSort();
    }

    public String getSort() {
        if (foundSorts.size() > 0) {
            return (String)foundSorts.get(0);
        }
        return null;
    }

    public String getId() {
        if (foundDefIds.size() > 0) {
            return (String)foundDefIds.get(0);
        }
        return null;
    }

    public Element getElem() {
        if (foundElems.size() > 0) {
            return (Element)foundElems.get(0);
        }
        return null;
    }

    private int currentPos = 0;
    private HashMap elems = new HashMap();

    private void checkNode(Node node) {
        if (node.getNodeType() == Node.TEXT_NODE) {
            String token = node.getNodeValue();

            if (isWithin(currentPos, currentPos + token.length() - 1)) {

                Element elem = (Element)node.getParentNode();
                String defid = elem.getAttribute(JXConstants.DefIdAttr);
                String sort = (String)elems.get(defid);

                if (elem.getNodeName().equals(JXConstants.KeywordElem)) {
                    Element e = (Element)elem.getParentNode();
                    if (e.getNodeName().equals(JXConstants.StatementElem)) {
                        defid = e.getAttribute(JXConstants.IdAttr);
                        sort = (String)elems.get(defid);
                    }
                }

                if (defid.length() != 0 && sort != null) {
                    foundSorts.add(sort);
                    foundDefIds.add(defid);
                    foundElems.add(elem);
                    // System.out.println("BINGO: " + token + " " + sort + " " + defid + " " + elem);
                }
            }
            currentPos = currentPos + token.length();

        } else if (node.getNodeType() == Node.ELEMENT_NODE) {
            Element elem = (Element)node;
        
            String sort = node.getNodeName();
            if (sort.equals(JXConstants.StatementElem)) {
                sort = elem.getAttribute(JXConstants.SortAttr);
            }
            String id = elem.getAttribute(JXConstants.IdAttr);
            if (id.length() != 0 && sort.length() != 0) {
                elems.put(id, sort);
            }

            NodeList nl = node.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                Node child = nl.item(i);
                checkNode(child);
            }
        }
    }

    private boolean isWithin(int begin, int end) {
        for (int pos = begin; pos <= end; pos++) {
            if (startPos <= pos && pos <= endPos) {
                return true;
            }
        }
        return false;
    }
}
