/*
 *     CFGCall.java  Dec 18, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.cfg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.wrapper.*;
import org.w3c.dom.*;
import java.util.ArrayList;
import org.w3c.dom.Element;

public class CFGCall extends CFGStatement {
    private String name;
    private ArrayList actualIns = new ArrayList();   // CFGParameter
    private ArrayList actualOuts = new ArrayList();  // CFGParameter
    private String calledType;

    private CFGCall() {
    }

    public CFGCall(int sort, Element elem) {
        super(sort, elem);
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void addActualIn(CFGParameter node) {
        actualIns.add(node);
        addUseVariables(node.getUseVars());
    }

    public ArrayList getActualIns() {
        return new ArrayList(actualIns);
    }

    public void addActualOut(CFGParameter node) {
        actualOuts.add(node);
        addDefVariables(node.getDefVars());
    }

    public ArrayList getActualOuts() {
        return new ArrayList(actualOuts);
    }

    public ArrayList getArguments() {
        return new ArrayList(actualIns);
    }

    public int getNumArguments() {
        return actualIns.size();
    }

    public CFGParameter getActualIn(int i) {
        return (CFGParameter)actualIns.get(i); 
    }

    public CFGParameter getActualOut(int i) {
        return (CFGParameter)actualOuts.get(i); 
    }

    public JavaVariable getReturnVariable() {
        if (hasArguments()) {
            return getActualOut(0).getDefVariable();
        }
        return null;
    }

    public boolean hasArguments() {
        return actualIns.size() != 0;
    }

    public void setCalledType(String type) {
        calledType = type;
    }

    public String getCalledType() {
        return calledType;
    }

    public boolean callSelf() {
        JavaCall ja = new JavaCall(elem);
        JavaMethod jm = new JavaMethod(elem);
        JavaClass ac = ja.getJavaClass();
        JavaClass mc = jm.getJavaClass(); 

        if (ac.equals(mc) && ja.getSignature().equals(jm.getSignature())) { 
            return true;
        }
        return false;
    }

    public void print(StringBuffer buf) {
        String mesg = "";
        switch (sort) {
            case GraphNodeSort.methodCall: mesg = "method call"; break;
            case GraphNodeSort.constructorCall: mesg = "constructor call"; break;
            case GraphNodeSort.polymorphicCall: mesg = "polymorphic call"; break;
        }
        mesg = mesg + " " + getName() + "()";
        printDefUse(mesg, buf);
    }
}
