/*
 *     CFGStatement.java  Dec 18, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.cfg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.wrapper.*;
import org.w3c.dom.Element;

public class CFGStatement extends CFGNode {
    protected JavaVariableList defs = new JavaVariableList();
    protected JavaVariableList uses = new JavaVariableList();

    protected CFGStatement() {
    }

    public CFGStatement(int sort, Element elem) {
        super(sort, elem);
    }

    public boolean isStatement() {
        return true;
    }

    public void setDefVariable(JavaStatement js) {
        defs = new JavaVariableList(js.getDefVariables());
    }

    public void setUseVariable(JavaStatement js) {
        uses = new JavaVariableList(js.getUseVariables());
    }

    public void setDefVariable(JavaVariable jv) {
        defs.clear();
        defs.add(jv);
    }

    public void setUseVariable(JavaVariable jv) {
        uses.clear();
        uses.add(jv);
    }

    public void addDefVariable(JavaVariable jv) {
        defs.add(jv);
    }

    public void addUseVariable(JavaVariable jv) {
        uses.add(jv);
    }

    public void addDefVariables(JavaVariableList jvl) {
        defs.addAll(jvl);
    }

    public void addUseVariables(JavaVariableList jvl) {
        uses.addAll(jvl);
    }

    public void removeDefVariable(JavaVariable jv) {
        defs.remove(jv);
    }

    public void removeUseVariable(JavaVariable jv) {
        uses.remove(jv);
    }

    public void clearDefVariable() {
        defs.clear();
    }

    public void clearUseVariable() {
        uses.clear();
    }

    public void addDefVariable(JavaStatement js) {
        defs.add(js.getDefVariables());
    }

    public void addUseVariable(JavaStatement js) {
        uses.add(js.getUseVariables());
    }

    public JavaVariableList getDefVars() {
        return new JavaVariableList(defs);
    }

    public JavaVariableList getUseVars() {
        return new JavaVariableList(uses);
    }

    public boolean containsDefVariable(JavaVariable jv) {
        return defs.contains(jv);
    }

    public boolean containsUseVariable(JavaVariable jv) {
        return uses.contains(jv);
    }

    public void print(StringBuffer buf) {
        String mesg = "";
        switch (sort) {
            case GraphNodeSort.switchSt: mesg = "switch"; break;
        }
        print(mesg, buf);
    }

    public void printDefUse(String m, StringBuffer buf) {
        String mesg = m + " "
            + " DEF={ " + defs.toString() + " }"
            + " USE={ " + uses.toString() + " }";
        print(mesg, buf);
    }
}
