/*
 *     GetCD.java  Dec 18, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.pdg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import java.util.Iterator;
 
public class GetCD {
    private PDG pdg;
    private CFG cfg;

    private GetCD() {
    }

    public GetCD(PDG pdg, CFG cfg) {
        this.pdg = pdg;
        this.cfg = cfg;
    }

    public void create() {
        Iterator it = cfg.getNodes().iterator();
        while (it.hasNext()) {
            CFGNode node = (CFGNode)it.next();

            if (node.isBranch()) {
                findCDs(node);
            }
        }
        findCDsAtEntry();

        pdg.setSrcDstNodes();
    }

    private void findCDsAtEntry() {
        CFGNode startNode = cfg.getStartNode();
        GetPostDominator postDominator = new GetPostDominator(cfg, startNode);

        Iterator it = postDominator.iterator();
        while (it.hasNext()) {
            CFGNode node = (CFGNode)it.next();

            if (node.isNormalStatement() || node.isFormalSt()) {
                CD edge = new CD(pdg.getNode(startNode), pdg.getNode(node));
                edge.setTrue();
                pdg.add(edge);
            }
        }
    }

    private void findCDs(CFGNode branchNode) {
        GetPostDominator postDominator = new GetPostDominator(cfg, branchNode);

        Iterator edgeIt = branchNode.getOutgoingEdges().iterator();
        while (edgeIt.hasNext()) {
            Flow branch = (Flow)edgeIt.next();

            CFGNode branchDstNode = (CFGNode)branch.getDstNode();
            GetPostDominator postDominatorL = new GetPostDominator(cfg, branchDstNode);
            postDominatorL.add(branchDstNode);

            Iterator nodeIt = cfg.getNodes().iterator();
            while (nodeIt.hasNext()) {
                CFGNode node = (CFGNode)nodeIt.next();

                if (node.isNormalStatement() && !branchNode.equals(node)
                    && !postDominator.contains(node) && postDominatorL.contains(node)) {

                    CD edge = new CD(pdg.getNode(branchNode), pdg.getNode(node));

                    if (branch.isTrue()) {
                        edge.setTrue();
                    } else if (branch.isFalse()) {
                        edge.setFalse();
                    } else {
                        edge.setFall();
                    }
                    pdg.add(edge);
                }
            }
        }
    }
}
