/*
 *     PDGNode.java  Dec 18, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.pdg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;

public class PDGNode extends GraphNode {
    protected CFGNode cfgNode;

    protected PDGNode() {
    }

    public PDGNode(CFGNode node) {
        super();
        cfgNode = node;
        sort = cfgNode.getSort();
        id = cfgNode.getId();
    }

    public CFGNode getCFGNode() {
        return cfgNode;
    }

    public String getJxId() {
        return cfgNode.getJxId();
    }

    public boolean isLoop() {
        return cfgNode.isLoop();
    }

    public boolean isBranch() {
        return cfgNode.isBranch();
    }

    public boolean isStatement() {
        return false;
    }

    public void print(String mesg, StringBuffer buf) {
        cfgNode.print(mesg, buf);
    }

 public void write(StringBuffer buf) {
        buf.append("    <node no=\"" + id + "\" id=\"" + getJxId() + "\"");
        buf.append("sort=\"" + sort + "\">\n");
        if (isStatement()) {
            writeStatement(buf, (PDGStatement)this);
        }
        buf.append("    </node>\n");
    }

    private void writeStatement(StringBuffer buf, PDGStatement jst) {
        if (jst.getDefVars().size() != 0) {
            buf.append("      <def>");
            writeVariables(buf, jst.getDefVars().iterator());
            buf.append("</def>\n");
        }
        if (jst.getUseVars().size() != 0) {
            buf.append("      <use>");
            writeVariables(buf, jst.getUseVars().iterator());
            buf.append("</use>\n");
        }
    }

    private void writeVariables(StringBuffer buf, Iterator it) {
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            buf.append("<var id=\"" + jv.getElemId() + "\"");
            buf.append(" name=\"" + jv.getName() + "\"/>");
        }
    }
}
