/*
 *     JavaClass.java  Dec 22, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.j2xml.XMLText;
import org.w3c.dom.*;
import java.util.*;

public class JavaClass extends JavaElement {
    private String name = null;
    private String fqn = null;
    private String access = null;
    private ArrayList methods = null;  // Node
    private ArrayList constructors = null;  // Node
    private ArrayList fields = null;  // Node
    private ArrayList sinits = null;  // Node

    protected JavaClass() {
    }

    public JavaClass(Node node) {
        super(node);
    }

    public String getName() {
        if (name == null) {
            name = getIdentName(elem);
        }
        return name;
    }

    public String getFqn() {
        if (fqn == null) {
            fqn = elem.getAttribute(JXConstants.FqnAttr);
        }
        return fqn;
    }

    public String getFqnOfAnonymous() {
        Node n = getFirstChildNode(getElem(), JXConstants.TypeElem);
        return ((Element)n).getAttribute(JXConstants.FqnAttr);
    }

    public String getElemId() {
        return getElemId(elem);
    }

    public boolean equals(JavaClass jc) {
        return jc != null && getFqn().equals(jc.getFqn());
    }

    public boolean isInterface() {
        return elem.getNodeName().equals(JXConstants.InterfaceElem);
    }

    public Node getSuperClassNode() {
        return getFirstChildNode(elem, JXConstants.ExtendsElem);
    }

    public ArrayList getSuperInterfaceNodes() {
        return getChildNodes(elem, JXConstants.ImplementsElem);
    }

    public ArrayList getMethodNodes() {
        if (methods == null) {
            Node node = getFirstChildNode(elem, JXConstants.MembersElem);
            methods = getChildNodes(node, JXConstants.MethodElem);
        }
        return methods;
    }

    public JavaMethodList getAllMethods() {
        JavaMethodList jmethods = new JavaMethodList();

        Iterator it = getMethodNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            jmethods.add(jm);
        }
        return jmethods;
    }

    public ArrayList getConstructorNodes() {
        if (constructors == null) {
            Node node = getFirstChildNode(elem, JXConstants.MembersElem);
            constructors = getChildNodes(node, JXConstants.ConstructorElem);
        }
        return constructors;
    }

    public JavaMethodList getAllConstructors() {
        JavaMethodList jmethods = new JavaMethodList();

        Iterator it = getConstructorNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            jmethods.add(jm);
        }
        return jmethods;
    }

    public JavaMethodList getMethodsAndConstructors() {
        JavaMethodList jmethods = new JavaMethodList();

        jmethods.addAll(getAllMethods());
        jmethods.addAll(getAllConstructors());
        return jmethods;
    }

    public ArrayList getMethodAndConstructorNodes() {
        ArrayList nodes = new ArrayList();

        nodes.addAll(getMethodNodes());
        nodes.addAll(getConstructorNodes());
        return nodes;
    }

    public ArrayList getFieldNodes() {
        if (fields == null) {
            Node node = getFirstChildNode(elem, JXConstants.MembersElem);
            fields = getChildNodes(node, JXConstants.FieldElem);
        }
        return fields;
    }

    public JavaFieldList getAllFields() {
        JavaFieldList jfields = new JavaFieldList();

        Iterator it = getFieldNodes().iterator();
        while (it.hasNext()) {
            JavaField jf = new JavaField((Node)it.next());
            jfields.add(jf);
        }
        return jfields;
    }

    public ArrayList getSinitNodes() {
        if (sinits == null) {
            Node node = getFirstChildNode(elem, JXConstants.MembersElem);
            sinits = getChildNodes(node, JXConstants.StaticInitElem);
        }
        return sinits;
    }

    private Element getSuperClassTypeNode() {
        NodeList nl = elem.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node node = nl.item(i);

            if (node.getNodeName().equals(JXConstants.ExtendsElem)) {
                Element e = (Element)getFirstChildNode(node, JXConstants.TypeElem);
                return e;
            }
        }
        return null;
    }

    public String getSuperClassName() {
        String superClassName;
        if (isAnonymous()) {
            superClassName = getFqnOfAnonymous();

        } else {
            Element e = getSuperClassTypeNode();

            if (e != null) {
                superClassName = e.getAttribute(JXConstants.FqnAttr);

            } else {
                if (isInterface()) {
                    superClassName = null;
                } else {
                    superClassName = "java.lang.Object";
                }
            }
        }
        return superClassName;
    }

    public String getSuperClassOriginalName() {
        String superClassName;
        Element e = getSuperClassTypeNode();

        if (e != null) {
            superClassName = getTypeName(e);
        } else {
            superClassName = "Object";
        }
        return superClassName;
    }

    public ArrayList getSuperInterfaceNames() {
        ArrayList superIntfNames = new ArrayList();

        NodeList nl = elem.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node node = nl.item(i);

            if (node.getNodeName().equals(JXConstants.ImplementsElem)) {
                Element e = (Element)getFirstChildNode(node, JXConstants.TypeElem);

                String superInterfaceName = e.getAttribute(JXConstants.FqnAttr);
                if (superInterfaceName != null) {
                    superIntfNames.add(superInterfaceName);
                }
            }
        }
        return superIntfNames;
    }

    public ArrayList getSuperInterfaceOriginalNames() {
        ArrayList superIntfNames = new ArrayList();
        NodeList nl = elem.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node node = nl.item(i);

            if (node.getNodeName().equals(JXConstants.ImplementsElem)) {
                Element e = (Element)getFirstChildNode(node, JXConstants.TypeElem);

                String superInterfaceName = getTypeName(e);
                if (superInterfaceName != null) {
                    superIntfNames.add(superInterfaceName);
                }
            }
        }
        return superIntfNames;
    }

    public boolean isInnerClass() {
        Node parent = elem.getParentNode();
        if (parent.getNodeName().equals(JXConstants.FileElem)) {
            return false;
        }
        return true;
    }

    public Node getOuterClassNode() {
        Node parent = elem.getParentNode();
        while (parent != null && !isClassOrInterfaceNode(parent)) {
            parent = parent.getParentNode();
        }
        return parent;
    }

    public ArrayList getOuterClassNodes() {
        ArrayList nodes = new ArrayList();
        Node parent = elem.getParentNode();

        while (parent != null) {
            if (isClassOrInterfaceNode(parent)) {
                nodes.add(parent);
            }
            parent = parent.getParentNode();
        }
        return nodes;
    }

    public JavaClassList getAllOuterClasses() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getOuterClassNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }
        return jclasses;
    }

    public JavaClassList getAllInnerClassesAndInterfaces() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getDescendantNodes(getElem(), JXConstants.ClassElem).iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }

        it = getDescendantNodes(getElem(), JXConstants.InterfaceElem).iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());
            jclasses.add(jc);
        }

        return jclasses;
    }

    public String getAccess() {
        if (access == null) {
            access = elem.getAttribute(JXConstants.AccessAttr);
            if (access.length() == 0) {
                access = JXConstants.Default;
            }
        }
        return access;
    }

    public JavaClassList getAncestors() {
        XMLText text = Repository.getXMLText(getJavaFile().getName());
        if (text != null) {
            return text.getAncestors(this);
        }
        return new JavaClassList();
    }

    public JavaClass getSuperClass() {
        Iterator it = getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass c = (JavaClass)it.next();

            if (!c.isInterface()) {
                int distance = getAncesOrDesceDistance(c.getFqn());
                if (distance == 1) {
                    return c;
                }
            }
        }
        return null;
    }

    public JavaClassList getSuperClasses() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass c = (JavaClass)it.next();

            int distance = getAncesOrDesceDistance(c.getFqn());
            if (distance == 1) {
                jclasses.add(c);
            }
        }
        return jclasses;
    }

    public JavaClassList getDescendants() {
        XMLText text = Repository.getXMLText(getJavaFile().getName());
        if (text != null) {
            return text.getDescendants(this);
        }
        return new JavaClassList();
    }

    public JavaClassList getSubClasses() {
        JavaClassList jclasses = new JavaClassList();

        Iterator it = getDescendants().iterator();
        while (it.hasNext()) {
            JavaClass c = (JavaClass)it.next();

            int distance = getAncesOrDesceDistance(c.getFqn());
            if (distance == 1) {
                jclasses.add(c);
            }
        }
        return jclasses;
    }

    public int getAncesOrDesceDistance(String fqn) {
        Iterator it = getDescendantNodes(elem, JXConstants.AncestorElem).iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            String name = elem.getAttribute(JXConstants.NameAttr);
            String dist = elem.getAttribute(JXConstants.DistanceAttr);
            if (fqn.equals(name)) {
                return Integer.parseInt(dist);
            }
        }

        it = getDescendantNodes(elem, JXConstants.DescendantElem).iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            String name = elem.getAttribute(JXConstants.NameAttr);
            String dist = elem.getAttribute(JXConstants.DistanceAttr);
            if (fqn.equals(name)) {
                return Integer.parseInt(dist);
            }
        }
        return 0;
    }

    public JavaMethod getJavaMethod(String signature) {
        Iterator it = getMethodNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            if (jm.getSignature().equals(signature)) {
                return jm;
            }
        }

        it = getConstructorNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            if (jm.getSignature().equals(signature)) {
                return jm;
            }
        }
        return null;
    }

    public Element getMethodNode(String id) {
        Iterator it = getMethodNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            if (id.equals(e.getAttribute(JXConstants.IdAttr))) {
                return e;
            }
        }

        it = getConstructorNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            if (id.equals(e.getAttribute(JXConstants.IdAttr))) {
                return e;
            }
        }
        return null;
    }        

    public JavaField getJavaField(String name) {
        Iterator it = getFieldNodes().iterator();
        while (it.hasNext()) {
            JavaField jf = new JavaField((Node)it.next());
            if (jf.getName().equals(name)) {
                return jf;
            }
        }
        return null;
    }

    public Element getFieldNode(String id) {
        Iterator it = getFieldNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            if (id.equals(e.getAttribute(JXConstants.IdAttr))) {
                return e;
            }
        }
        return null;
    }        

    public ArrayList getUsedTypes() {
        ArrayList types = new ArrayList();
        NodeList nl = elem.getElementsByTagName(JXConstants.TypeElem);
        for (int i = 0; i < nl.getLength(); i++) {
            Element e = (Element)nl.item(i);
            if (getElemSort(e).equals(JXConstants.ObjectRef)) {
                String name = getIdentName(e);

                int index = name.indexOf(".");
                if (index != -1) {
                    name = name.substring(0, index);
                }
                types.add(name);
            }
        }
        return types;
    }

    public Node getIdentNode() {
        return getFirstChildNode(elem, JXConstants.IdentElem);
    }

    public Node getExtendsIdentNode() {
        Element superElem = (Element)getSuperClassNode();
        Element e = (Element)getFirstChildNode(superElem, JXConstants.TypeElem);
        return getFirstChildNode(e, JXConstants.IdentElem);
    }

    public String getExtendsIdentName() {
        return getExtendsIdentNode().getFirstChild().getNodeValue();
    }

    public Node getLastMemeberNode() {
        Element e = (Element)getFirstChildNode(elem, JXConstants.MembersElem);
        return e.getLastChild();
    }

    public boolean isChildOf(JavaClass jc) {
        if (getSuperClassName().equals(jc.getFqn())) {
            return true;
        }
        return false;
    }

    public boolean isPublic() {
        return getAccess().equals(JXConstants.Public);
    }

    public boolean isProtected() {
        return getAccess().equals(JXConstants.Protected);
    }

    public boolean isPrivate() {
        return getAccess().equals(JXConstants.Private);
    }

    public boolean isDefault() {
        return getAccess().equals(JXConstants.Default);
    }

    public boolean isFinal() {
        return elem.getAttribute(JXConstants.FinalAttr).equals(JXConstants.YES);
    }

    public boolean isAbstract() {
        if (isInterface()) {
            return true;
        }
        return elem.getAttribute(JXConstants.AbstractAttr).equals(JXConstants.YES);
    }

    public boolean isStatic() {
        return elem.getAttribute(JXConstants.StaticAttr).equals(JXConstants.YES);
    }

    public boolean isStrictfp() {
        return elem.getAttribute(JXConstants.StrictfpAttr).equals(JXConstants.YES);
    }

    public String getModifier() {
        StringBuffer buf = new StringBuffer();

        if (isPublic()) {
            buf.append(" public");
        } else if (isProtected()) {
            buf.append(" protected");
        } else if (isPrivate()) {
            buf.append(" private");
        }

        if (isFinal()) {
            buf.append(" final");
        }
        if (isAbstract()) {
            buf.append(" abstract");
        }
        if (isStatic()) {
            buf.append(" static");
        }
        if (isStrictfp()) {
            buf.append(" strictfp");
        }

        if (buf.length() != 0) {
            return buf.substring(1);
        }
        return "";
    }

    public boolean isAnonymous() {
        return getIdentName(elem).length() == 0;
    }

    public void print(StringBuffer buf) {
        print("", buf);
    }

    public void print(String indent, StringBuffer buf) {
        if (isInterface()) {
            buf.append(indent + "INTERFACE: ");
        } else {
            buf.append(indent + "CLASS: ");
        }
        buf.append(getAccess() + " " + getName() + "[" + getFqn() + "]\n");
        buf.append(indent + "EXTENDS: " + getSuperClassName() + "\n");
        Iterator it = getSuperInterfaceNames().iterator();
        while (it.hasNext()) {
            buf.append(indent + "IMPLEMENTS: " + (String)it.next() + "\n");
        }

        printAllFields(indent, buf);
        printAllConstructors(indent, buf);
        printAllMethods(indent, buf);
    }        

    public void printAllMethods(String indent, StringBuffer buf) {
        Iterator it = getMethodNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            jm.print(indent + "  ", buf);
        }
    }

    public void printAllConstructors(String indent, StringBuffer buf) {
        Iterator it = getConstructorNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            jm.print(indent + "  ", buf);
        }
    }

    public void printAllFields(String indent, StringBuffer buf) {
        Iterator it = getFieldNodes().iterator();
        while (it.hasNext()) {
            JavaField jf = new JavaField((Node)it.next());
            jf.print(indent + "  ", buf);
        }
    }
}
