/*
 *     JavaClassList.java  Dec 12, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.w3c.dom.*;
import java.util.*;

public class JavaClassList {
    private ArrayList list = new ArrayList();  // JavaClass

    public JavaClassList() {
    }

    public JavaClassList(List classes) {
        this(classes.iterator());
    }

    public JavaClassList(JavaClassList classes) {
        this(classes.iterator());
    }

    private JavaClassList(Iterator it) {
        while (it.hasNext()) {
            list.add(it.next());
        }
    }

    public void clear() {
        list.clear();
    }

    public void add(JavaClass jc) {
        if (jc != null && !list.contains(jc)) {
            list.add(jc);
        }
    }

    public void add(int index, JavaClass jc) {
        if (jc != null && !list.contains(jc)) {
            list.add(index, jc);
        }
    }

    public void addAll(JavaClassList l) {
        Iterator it = l.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            add(jc);
        }
    }

    public void remove(JavaClass jc) {
        if (list.contains(jc)) {
            list.remove(jc);
        }
    }

    public boolean contains(JavaClass jc) {
        return getJavaClass(jc) != null;
    }

    public JavaClass getJavaClass(JavaClass jc) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaClass c = (JavaClass)it.next();
            if (jc.equals(c)) {
                return c;
            }
        }
        return null;
    }

    public JavaClass getJavaClass(String name) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaClass c = (JavaClass)it.next();
            if (name.equals(c.getFqn())) {
                return c;
            }
        }
        return null;
    }

    public JavaClass get(int i) {
        if (i < list.size()) {
            return (JavaClass)list.get(i);
        }
        return null;
    }

    public JavaClass getFirstClass() {
        if (list.size() != 0) {
            return (JavaClass)list.get(0);
        }
        return null;
    }

    public JavaClass getLast() {
        if (list.size() != 0) {
            return (JavaClass)list.get(list.size() - 1);
        }
        return null;
    }

    public JavaClass removeFirst() {
        if (list.size() != 0) {
            return (JavaClass)list.remove(0);
        }
        return null;
    }

    public JavaClass removeLast() {
        if (list.size() != 0) {
            return (JavaClass)list.remove(list.size() - 1);
        }
        return null;
    }

    public boolean isEmpty() {
        if (list.size() == 0) {
            return true;
        }
        return false;
    }

    public int size() {
        return list.size();
    }

    public Iterator iterator() {
        return list.iterator();
    }

    public ArrayList getArrayList() {
        ArrayList classes = new ArrayList();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            classes.add(jc);
        }
        return classes;
    }

    public void print() {
        System.out.println("CLASSES = " + toString());
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("[");
        Iterator it = list.iterator();
        while (it.hasNext()) {
            buf.append(" ");
            JavaClass jc = (JavaClass)it.next();
            buf.append(jc.getFqn());
        }
        buf.append(" ]");
        return buf.toString();
    }
}
