/*
 *     JavaExpr.java  Dec 13, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.TypeConversion;
import org.w3c.dom.*;
import java.util.*;

public class JavaExpr extends JavaElement {
    protected String sort;
    protected JavaClass jclass;
    protected JavaMethod jmethod;
    private ArrayList exprs = new ArrayList();  // Node

    protected JavaExpr() {
    }

    public JavaExpr(Node node) {
        super(node);

        sort = getElemSort(elem);
        jclass = getJavaClass(elem);
        jmethod = getJavaMethod(elem);
    }

    public String getName() {
        return getName(elem);
    }

    public String getName(Element prefix) {
        if (prefix.getNodeName().equals(JXConstants.QNameElem)) {
            return getIdentName(prefix);

        } else if (prefix.getNodeName().equals(JXConstants.KeywordElem)) {
            return prefix.getFirstChild().getNodeValue();
        }

        String sort = getElemSort(prefix);
        if (sort.equals(JXConstants.VarRef)) {
            return getIdentName(prefix);

        } else if (sort.equals(JXConstants.MethodCall)) {
            return getIdentName(prefix);
        }

        return null;
    }

    public boolean isAssign() {
        return sort.equals(JXConstants.Assignment);
    }

    public boolean isOpAssign() {
        return !sort.equals(JXConstants.Assignment) &&
                sort.startsWith(JXConstants.Assignment);
    }

    public boolean isPreOp() {
        return isPreIncOp() || isPreDecOp();
    }

    public boolean isPreIncOp() {
        return sort.equals(JXConstants.PreInc);
    }

    public boolean isPreDecOp() {
        return sort.equals(JXConstants.PreDec);
    }

    public boolean isPostOp() {
        return isPostIncOp() || isPostDecOp();
    }

    public boolean isPostIncOp() {
        return sort.equals(JXConstants.PostInc);
    }

    public boolean isPostDecOp() {
        return sort.equals(JXConstants.PostDec);
    }
    
    public boolean isVarRef() {
        return sort.equals(JXConstants.VarRef);
    }

    public boolean isDotOp() {
        return sort.equals(JXConstants.DotOp);
    }

    public boolean isParen() {
        return sort.equals(JXConstants.ParenOp);
    }

    public boolean isConstructorCall() {
        return sort.equals(JXConstants.ConstructorCall);
    }

    public boolean isSuperConstructorCall() {
        return sort.equals(JXConstants.SuperConstructorCall);
    }

    public boolean isMethodCall() {
        return sort.equals(JXConstants.MethodCall);
    }

    public boolean isInstanceCreation() {
        return sort.equals(JXConstants.InstanceCreation);
    }

    public boolean isArrayCreation() {
        return sort.equals(JXConstants.ArrayCreation);
    }

    public boolean isInSimpleAssignment() {
        if (isAssign() || isOpAssign() || isPreOp() || isPostOp()) {
            Node node = elem.getParentNode();

            if (node.getNodeName().equals(JXConstants.StatementElem)) {
                Element e = (Element)node;
                if (e.getAttribute(JXConstants.SortAttr).equals(JXConstants.Expression)) {
                    return true;
                }
            }
        }
        return false;
    }

    public String getDotPrefixType(Element prefix) {
        if (prefix != null) {
            return getTypeFqn(prefix);
        }
        return null;
    }        

    public Node getDotPrefix() {
        Element prefix = null;
        if (isDotChild()) {
            prefix = (Element)getPrefixNode(elem);

            while (prefix != null &&
                   (getElemSort(prefix).equals(JXConstants.DotOp) ||
                    getElemSort(prefix).equals(JXConstants.ParenOp))) {
                prefix = (Element)getLastNodeInExpr(prefix);
            }

            while (prefix != null && getElemSort(prefix).equals(JXConstants.ArrayAccess)) {
                prefix = (Element)getFirstNodeInArrayAccess(prefix);
            }
            
            if (prefix != null) {
                ArrayList nodes = new ArrayList();

                if (prefix.getNodeName().equals(JXConstants.TypeElem)) {
                    nodes.add(prefix);

                } else if (prefix.getNodeName().equals(JXConstants.KeywordElem)) {
                    String keyword = prefix.getFirstChild().getNodeValue();
                    if (keyword.equals("super")) {
                        nodes.add(prefix);
                    }

                } else {
                    collectValidNodesInExpr(prefix, nodes);
                }

                if (nodes.size() != 0) {
                    prefix = (Element)nodes.get(0);
                }
            }
        }
        return prefix;
    }

    private boolean isDotChild() {
        Node parent = elem.getParentNode();
        if (parent != null) {
            if (parent.getNodeName().equals(JXConstants.ExpressionElem)) {
                if (getElemSort((Element)parent).equals(JXConstants.DotOp)) {
                    return true;
                }
            }
        }
        return false;
    }

    private Node getPrefixNode(Node node) {
        Element prefix = (Element)getPreviousSiblingNode(node);
        while (prefix != null && !prefix.getNodeName().equals(JXConstants.OperatorElem)) {
            prefix = (Element)getPreviousSiblingNode(prefix);
        }

        if (prefix != null) {
            prefix = (Element)getPreviousSiblingNode(prefix);
            if (prefix != null) {
                return prefix;
            }
        }
        return null;
    }

    private Node getPreviousSiblingNode(Node node) {
        Node prev = node.getPreviousSibling();
        if (prev != null && prev.getNodeType() == Node.ELEMENT_NODE) {
            return prev;
        }
        return null;
    }

    private Node getLastNodeInExpr(Node node) {
        ArrayList nodes = getChildNodes(node, JXConstants.ExpressionElem);
        if (nodes.size() != 0) {
            return (Node)nodes.get(nodes.size() - 1);
        }
        return null;
    }

    private Node getFirstNodeInArrayAccess(Node node) {
        ArrayList nodes = getChildNodes(node, JXConstants.ExpressionElem);
        if (nodes.size() != 0) {
            return (Node)nodes.get(0);
        }
        return null;
    }

    protected void collectValidNodesInExpr(Node node, ArrayList nodes) {
        if (isIndexExpr(node) || isBlockExpr(node)) {
            return;
        }

        if (node.getNodeType() == Node.ELEMENT_NODE) {
            Element e = (Element)node;
            
            if (e.getNodeName().equals(JXConstants.TypeElem)) {
                nodes.add(e);

            } else if (e.getNodeName().equals(JXConstants.KeywordElem)) {
                String keyword = e.getFirstChild().getNodeValue();
                if (keyword.equals("super")) {
                    nodes.add(e);
                }
                
            } else if (e.getNodeName().equals(JXConstants.ExpressionElem)) {
                String sort = ((Element)e).getAttribute(JXConstants.SortAttr);

                if (sort.equals(JXConstants.VarRef) ||
                    sort.equals(JXConstants.Literal) ||
                    sort.equals(JXConstants.ThisRef) ||
                    sort.equals(JXConstants.MethodCall) ||
                    sort.equals(JXConstants.ConstructorCall) ||
                    sort.equals(JXConstants.InstanceCreation) ||
                    sort.equals(JXConstants.InstanceOf) ||
                    sort.equals(JXConstants.Cast) ||
                    sort.startsWith(JXConstants.ConditionOp)) {
                    nodes.add(e);

                } else if (sort.equals(JXConstants.ClassRef)) {
                    Element ke = (Element)e.getFirstChild();
                    String keyword = ke.getFirstChild().getNodeValue();
                    if (keyword.equals("class")) {
                        nodes.add(ke);
                    }
                }

                if (sort.equals(JXConstants.DotOp) || sort.equals(JXConstants.ParenOp)) {
                    JavaExpr je = new JavaExpr(e);
                    Node child = je.getLastNodeInExpr(e);
                    collectValidNodesInExpr(child, nodes);
                    
                } else if (!sort.equals(JXConstants.MethodCall) &&
                           !sort.equals(JXConstants.ConstructorCall) &&
                           !sort.equals(JXConstants.SuperConstructorCall) &&
                           !sort.equals(JXConstants.InstanceOf) &&
                           !sort.equals(JXConstants.Cast) &&
                           !sort.startsWith(JXConstants.ConditionOp)) {
                    NodeList nl = e.getChildNodes();
                    for (int i = 0; i < nl.getLength(); i++) {
                        Node child = nl.item(i);
                        collectValidNodesInExpr(child, nodes);
                    }
                }
            }
        }
    }

    private boolean isIndexExpr(Node node) {
        Node n = getSiblingOpNode(node, "[");
        if (n != null && getDescendantNodes(elem, JXConstants.OperatorElem).contains(n)) {
            return true;
        }
        return false;
    }

    private boolean isBlockExpr(Node node) {
        Node n = getSiblingOpNode(node, "{");
        if (n != null && getDescendantNodes(elem, JXConstants.OperatorElem).contains(n)) {
            return true;
        }
        return false;
    }

    private Node getSiblingOpNode(Node node, String op) {
        Node prev = node.getPreviousSibling();
        while (prev != null) {
            if (prev.getNodeType() == Node.ELEMENT_NODE) {
                Element e = (Element)prev;
                if (e.getNodeName().equals(JXConstants.OperatorElem)) {

                    Node n = prev.getFirstChild();
                    if (n.getNodeType() == Node.TEXT_NODE) {
                        if (n.getNodeValue().equals(op)) {
                            return e;
                        }
                    }
                }
            }
            prev = prev.getPreviousSibling();
        }
        return null;
    }

    protected String getTypeFqn(Element node) {
        String type = "";

        if (node.getNodeName().equals(JXConstants.TypeElem)) {

            Element n = node;
            String nid = n.getAttribute(JXConstants.TypeNextAttr);
            String array = "";
            while (nid.length() != 0) {
                String sort = n.getAttribute(JXConstants.SortAttr);
                if (sort.equals(JXConstants.Array)) {
                    array = array + "[]";
                }

                JavaElement je = new JavaElement(n);
                NodeList nl = je.getJavaFile().getElem().getElementsByTagName(JXConstants.TypeElem);
                n = je.getElemFromId(nl, nid);
                nid = n.getAttribute(JXConstants.TypeNextAttr);
            }

            if (n.getAttribute(JXConstants.SortAttr).equals(JXConstants.Primitive)) {
                type = n.getFirstChild().getFirstChild().getNodeValue();
            } else {
                type = n.getAttribute(JXConstants.FqnAttr);
            }
            type = type + array;

        } else if (node.getNodeName().equals(JXConstants.KeywordElem)) {
            String keyword = node.getFirstChild().getNodeValue();
            if (keyword.equals("super")) {
                type = getJavaClass().getSuperClassName();

            } else if (keyword.equals("class")) {
                type = "java.lang.Class";
            }

        } else if (node.getNodeName().equals(JXConstants.ExpressionElem)) {
            String sort = node.getAttribute(JXConstants.SortAttr);

            if (sort.equals(JXConstants.VarRef)) {
                JavaVariable jv = new JavaVariable(node);
                if (jv != null) {
                    Element d = (Element)jv.getDeclaration();
                    type = jv.getTypeFqn();
                }
            
            } else if (sort.equals(JXConstants.Literal)) {
                Element e = (Element)node.getFirstChild();
                sort = e.getAttribute(JXConstants.SortAttr);
                type = TypeConversion.getLiteralType(sort);

            } else if (sort.equals(JXConstants.ThisRef)) {
                type = getJavaClass().getFqn();

            } else if (sort.equals(JXConstants.MethodCall)) {
                JavaCall ja = new JavaCall(node);
                type = ja.getTypeFqn();
                
            } else if (sort.equals(JXConstants.ConstructorCall)) {
                JavaConstructorCall ja = new JavaConstructorCall(node);
                type = ja.getTypeFqn();

            } else if (sort.equals(JXConstants.InstanceCreation)) {
                JavaElement je = new JavaElement(node);

                Node n = je.getFirstChildNode(node, JXConstants.ClassElem);
                if (n.getNodeName().equals(JXConstants.ClassElem)) {
                    JavaClass jc = new JavaClass(n);
                    if (jc.isAnonymous()) {
                        
                        Element e = (Element)n.getFirstChild();
                        if (e.getNodeName().equals(JXConstants.TypeElem)) {
                            type = e.getAttribute(JXConstants.FqnAttr);
                        }
                    }

                } else {
                    n = je.getFirstChildNode(node,JXConstants.ExpressionElem);
                    if (n.getNodeName().equals(JXConstants.ExpressionElem)) {
                        Element e = (Element)n.getFirstChild();
                        if (e.getNodeName().equals(JXConstants.TypeElem)) {
                            type = e.getAttribute(JXConstants.FqnAttr);
                        }
                    }
                }
                
            } else if (sort.startsWith(JXConstants.ConditionOp)) {
                type = TypeConversion.getLiteralType("BOOL");

            } else if (sort.startsWith(JXConstants.InstanceOf)) {
                type = TypeConversion.getLiteralType("BOOL");

            } else if (sort.equals(JXConstants.Cast)) {
                JavaElement je = new JavaElement(node);

                Node n = je.getFirstChildNode(node, JXConstants.ExpressionElem);
                if (n.getNodeName().equals(JXConstants.ExpressionElem)) {
                    Element e = (Element)n.getFirstChild();

                    if (e.getNodeName().equals(JXConstants.TypeElem)) {
                        String fqn = e.getAttribute(JXConstants.FqnAttr);
                        if (fqn.length() != 0) {
                            type = fqn;

                        } else {
                            e = (Element)e.getFirstChild();
                            if (e.getNodeName().equals(JXConstants.KeywordElem)) {
                                type = e.getFirstChild().getNodeValue();
                            }
                        }
                    }
                }
            }

            int num = getNumOfArrayAccess(node);
            for (int i = 0; i < num; i++) {
                int sep = type.lastIndexOf("[");
                if (sep != -1) {
                    type = type.substring(0, sep);
                }
            }
        }

        return type;
    }

    private int getNumOfArrayAccess(Node node) {
        Node parent = node.getParentNode();
        while (isDotAccess(parent)) {
            parent = parent.getParentNode();
        }

        int num = 0;
        while (isArrayAccess(parent)) {
            num++;

            parent = parent.getParentNode();
            while (isDotAccess(parent)) {
                parent = parent.getParentNode();
            }
        }
        return num;
    }

    private boolean isArrayAccess(Node node) {
        if (node.getNodeType() == Node.ELEMENT_NODE &&
            node.getNodeName().equals(JXConstants.ExpressionElem)) {

            String sort = ((Element)node).getAttribute(JXConstants.SortAttr);
            if (sort.equals(JXConstants.ArrayAccess)) {
                return true;
            }
        }
        return false;
    }

    private boolean isDotAccess(Node node) {
        if (node.getNodeType() == Node.ELEMENT_NODE &&
            node.getNodeName().equals(JXConstants.ExpressionElem)) {

            String sort = ((Element)node).getAttribute(JXConstants.SortAttr);
            if (sort.equals(JXConstants.DotOp)) {
                return true;
            }
        }
        return false;
    }

    public JavaClass getJavaClass() {
        return jclass;
    }

    public JavaMethod getJavaMethod() {
        return jmethod;
    }

    public void print(StringBuffer buf) {
        print("", buf);
    }        

    public void print(String indent, StringBuffer buf) {
        Iterator it = exprs.iterator(); 
        while (it.hasNext()) {
            Element e = (Element)it.next();
            buf.append(indent + e.getNodeName() + " " + e.getAttribute(JXConstants.SortAttr));
            buf.append("\n");
        }
    }
}
