/*
 *     JavaLocal.java  Dec 13, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.w3c.dom.*;
import java.util.*;

public class JavaLocal extends JavaStatement {
    private String name = null;
    private String type = null;
    private String fqn = null;
    private JavaVariable jv;
    private JavaClass jclass;
    private JavaMethod jmethod;

    protected JavaLocal() {
    }

    public JavaLocal(Node node) {
        super(node);

        jclass = getJavaClass(elem);
        jmethod = getJavaMethod(elem);
        jv = new JavaVariable(node, elem);
        addDefVariableFirst(jv);
    }

    public String getName() {
        if (name == null) {
            name = getIdentName(elem);
        }
        return name;
    }

    public String getType() {
        if (type == null) {
            type = getElemType(elem);
        }
        return type;
    }

    public String getTypeFqn() {
        if (fqn == null) {
            fqn = getElemFqn(elem);
        }
        return fqn;
    }

    public boolean equals(JavaLocal jl) {
        return jl != null && isInTheSameMethod(jl) && getName().equals(jl.getName());
    }

    public boolean isFinal() {
        return elem.getAttribute(JXConstants.FinalAttr).equals(JXConstants.YES);
    }

    public boolean isVolatile() {
        return elem.getAttribute(JXConstants.VolatileAttr).equals(JXConstants.YES);
    }

    public boolean isOmitted() {
        return elem.getAttribute(JXConstants.OmittedAttr).equals(JXConstants.YES);
    }

    public JavaClass getJavaClass() {
        return jclass;
    }

    public JavaMethod getJavaMethod() {
        return jmethod;
    }

    public JavaVariable getJavaVariable() {
        return jv;
    }

    public boolean isParameter() {
        return false;
    }

    public boolean isLocal() {
        return true;
    }

    public Node getIdentNode() {
        return getFirstChildNode(elem, JXConstants.IdentElem);
    }

    public int getNumLocalInDeclaration() {
        String typeId = getTypeNode(elem).getAttribute(JXConstants.IdAttr);

        int num = 0;
        Iterator it = jmethod.getLocalNodes().iterator();
        while (it.hasNext()) {
            Element e = (Element)it.next();
            if (e.getAttribute(JXConstants.TypeFirstAttr).equals(typeId)) {
                num++;
            }
        }
        return num;
    }

    public void print(StringBuffer buf) {
        print("", buf);
    }

    public void print(String indent, StringBuffer buf) {
        buf.append(indent + "LOCAL: " + getName());
        buf.append("@" + getType() + "[" + getTypeFqn() + "]");
        buf.append(" = " + uses.toString());
    }        
}
