/*
 *     JavaMethodList.java  Dec 12, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.w3c.dom.*;
import java.util.*;

public class JavaMethodList {
    private ArrayList list = new ArrayList();  // JavaMethod

    public JavaMethodList() {
    }

    public JavaMethodList(List methods) {
        this(methods.iterator());
    }

    public JavaMethodList(JavaMethodList methods) {
        this(methods.iterator());
    }

    private JavaMethodList(Iterator it) {
        while (it.hasNext()) {
            list.add(it.next());
        }
    }

    public void clear() {
        list.clear();
    }

    public void add(JavaMethod jm) {
        if (jm != null && !list.contains(jm)) {
            list.add(jm);
        }
    }

    public void add(int index, JavaMethod jm) {
        if (jm != null && !list.contains(jm)) {
            list.add(index, jm);
        }
    }

    public void addAll(JavaMethodList l) {
        Iterator it = l.iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();
            add(jm);
        }
    }

    public void remove(JavaMethod jm) {
        if (list.contains(jm)) {
            list.remove(jm);
        }
    }

    public boolean contains(JavaMethod jm) {
        return getJavaMethod(jm) != null;
    }

    public JavaMethod getJavaMethod(JavaMethod jm) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaMethod m = (JavaMethod)it.next();
            if (jm.equals(m)) {
                return m;
            }
        }
        return null;
    }

    public JavaMethod getJavaMethod(String sig) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaMethod m = (JavaMethod)it.next();
            if (sig.equals(m.getSignature())) {
                return m;
            }
        }
        return null;
    }

    public JavaMethod get(int i) {
        if (i < list.size()) {
            return (JavaMethod)list.get(i);
        }
        return null;
    }

    public boolean isEmpty() {
        if (list.size() == 0) {
            return true;
        }
        return false;
    }

    public int size() {
        return list.size();
    }

    public Iterator iterator() {
        return list.iterator();
    }

    public ArrayList getArrayList() {
        ArrayList methods = new ArrayList();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();
            methods.add(jm);
        }
        return methods;
    }

    public void print() {
        System.out.println("METHODS = " + toString());
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("[");
        Iterator it = list.iterator();
        while (it.hasNext()) {
            buf.append(" ");
            JavaMethod jm = (JavaMethod)it.next();
            buf.append(jm.getSignature());
        }
        buf.append(" ]");
        return buf.toString();
    }
}
