/*
 *     JavaVariable.java  Dec 19, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.xml.dom.*;
import org.w3c.dom.*;

public class JavaVariable extends JavaExpr {
    protected Element decl = null;
    private JavaClass jclass;
    private JavaMethod jmethod;

    protected JavaVariable() {
    }

    public JavaVariable(Node node, Element decl) {
        super(node);

        jclass = getJavaClass(elem);
        jmethod = getJavaMethod(elem);
        this.decl = decl;
    }

    public JavaVariable(Node node) {
        super(node);

        jclass = getJavaClass(elem);
        jmethod = getJavaMethod(elem);

        if (getId() != null) {

            JavaFile jf = getJavaFile();
            JavaClass jc;

            String ref = getRefClass();
            if (ref.length() == 0) {
                jc = getJavaClass();
            } else {
                int sep = ref.indexOf("$");
                if (sep != -1) {
                    ref = ref.substring(0, sep);
                }

                jc = Repository.getJavaClass(ref, getJavaFile());
            }

            String defid = getId();
            if (jc != null && jf.getName().equals(jc.getJavaFile().getName())) {
                NodeList nl = jf.getElem().getElementsByTagName(JXConstants.FieldElem);
                decl = getElemFromId(nl, defid);

                if (decl == null) {
                    nl = jf.getElem().getElementsByTagName(JXConstants.LocalElem);
                    decl = getElemFromId(nl, defid);
                }

                if (decl == null) {
                    nl = jf.getElem().getElementsByTagName(JXConstants.ParameterElem);
                    decl = getElemFromId(nl, defid);
                }
            }
        }
    }

    public String getId() {
        Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
        return getElemId(e);
    }

    public String getElemId() {
        return getElemId(getElem());
    }

    public String getName() {
        return getIdentName(elem);
    }

    public JavaClass getJavaClass() {
        return jclass;
    }

    public JavaMethod getJavaMethod() {
        return jmethod;
    }

    public Node getStatementNode() {
        Node node = getElem();
        while (node != null && !isStatementNode(node)) {
            node = node.getParentNode();
        }
        return node;
    }

    public String getCastType() {
        Element parent = (Element)elem.getParentNode();
        if (getElemSort(parent).equals(JXConstants.Cast)) {

            Element exp = (Element)getFirstChildNode(parent, JXConstants.ExpressionElem);
            if (getElemSort(exp).equals(JXConstants.TypeRef)) {

                Element e = (Element)getFirstChildNode(exp, JXConstants.TypeElem);
                String fqn = e.getAttribute(JXConstants.FqnAttr);
                if (fqn.length() == 0) {
                    fqn = getTypeName(e);
                }
                return fqn;
            }
        }
        return null;
    }

    public String getTypeFqn() {
        String cast = getCastType();
        if (cast != null) {
            return cast;
        }

        if (decl != null) {
            return getElemFqn(decl);

        } else {
            Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
            return e.getAttribute(JXConstants.FqnAttr);
        }
    }

    public Node getDeclaration() {
        return decl;
    }

    public String getRefClass() {
        Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
        return e.getAttribute(JXConstants.RefAttr);
    }

    public String getRefFieldId() {
        String defid;
        Element e = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
        if (e.getAttribute(JXConstants.RefAttr).length() != 0) {
            defid = getElemId(e);
        } else {
            defid = getElemId(elem);
        }
        return defid;
    }

    public Node getUsedFieldNode() {
        String defid = getRefFieldId();
        if (defid.startsWith("s")) {
            JavaClass jc = Repository.getJavaClass(getRefClass(), getJavaFile());
            if (jc != null) {
                return jc.getFieldNode(defid);
            }
        }
        return null;
    }

    public boolean equals(JavaVariable jv) {
        if (this == jv) {
            return true;
        }
        return getId().equals(jv.getId());
    }

    public boolean isPrimitive() {
        Element e = getTypeNode(elem);
        String sort = getElemSort(e);
        if (sort.equals(JXConstants.Primitive)) {
            return true;
        }
        return false;
    }

    public boolean isDefVariable() {
        return elem.getAttribute(JXConstants.WriteAttr).equals(JXConstants.YES);
    }

    public boolean isUseVariable() {
        return elem.getAttribute(JXConstants.ReadAttr).equals(JXConstants.YES);
    }

    public boolean isRefVariable() {
        return !isDefVariable() && !isUseVariable();
    }

    public boolean isField() {
        if (decl != null) {
            return isFieldNode(decl);
        }
        return true;
    }        

    public boolean isLocal() {
        if (decl != null) {
            return isLocalNode(decl);
        }
        return false;
    }        

    public boolean isParameter() {
        if (decl != null) {
            return isParameterNode(decl);
        }
        return false;
    }        

    public Node getIdentNode() {
        return getFirstChildNode(elem, JXConstants.IdentElem);
    }

    public void print(StringBuffer buf) {
        buf.append(getName() + "@" + getTypeFqn() + "\n");
    }
}
