/*
 *     JavaVariableList.java  Dec 13, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.w3c.dom.*;
import java.util.*;

public class JavaVariableList {
    private ArrayList list = new ArrayList();  // JavaVariable

    public JavaVariableList() {
    }

    public JavaVariableList(JavaVariableList jvl) {
        Iterator it = jvl.iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            list.add(jv);
        }
    }

    public void clear() {
        list.clear();
    }

    public void add(JavaVariable jv) {
        if (jv != null && !list.contains(jv)) {
            list.add(jv);
        }
    }

    public void add(int index, JavaVariable jv) {
        if (jv != null && !list.contains(jv)) {
            list.add(index, jv);
        }
    }

    public void add(JavaVariableList jl) {
        Iterator it = jl.iterator();
        while (it.hasNext()) {
            add((JavaVariable)it.next());
        }
    }

    public void addAll(JavaVariableList jl) {
        add(jl);
    }

    public void remove(JavaVariable jv) {
        if (list.contains(jv)) {
            list.remove(jv);
        }
    }

    public boolean contains(JavaVariable jv) {
        return getJavaVariable(jv) != null;
    }

    public boolean strictlyContains(JavaVariable jv) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaVariable v = (JavaVariable)it.next();
            if (jv == v) {
                return true;
            }
        }
        return false;
    }

    private JavaVariable getJavaVariable(JavaVariable jv) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaVariable v = (JavaVariable)it.next();
            if (jv.equals(v)) {
                return v;
            }
        }
        return null;
    }

    public JavaVariable getJavaVariable(String name) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaVariable v = (JavaVariable)it.next();
            if (name.equals(v.getName())) {
                return v;
            }
        }
        return null;
    }

    public JavaVariable get(int i) {
        if (i < list.size()) {
            return (JavaVariable)list.get(i);
        }
        return null;
    }

    public JavaVariable getFirst() {
        if (list.size() != 0) {
            return (JavaVariable)list.get(0);
        }
        return null;
    }

    public JavaVariable getLast() {
        if (list.size() != 0) {
            return (JavaVariable)list.get(list.size() - 1);
        }
        return null;
    }

    public JavaVariable removeFirst() {
        if (list.size() != 0) {
            return (JavaVariable)list.remove(0);
        }
        return null;
    }

    public JavaVariable removeLast() {
        if (list.size() != 0) {
            return (JavaVariable)list.remove(list.size() - 1);
        }
        return null;
    }

    public boolean isEmpty() {
        if (list.size() == 0) {
            return true;
        }
        return false;
    }

    public int size() {
        return list.size();
    }

    public Iterator iterator() {
        return list.iterator();
    }

    public void print() {
        System.out.println("VARS = " + toString());
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            buf.append(" ");
            buf.append(jv.getName());
            buf.append("@");
            buf.append(jv.getTypeFqn());
        }
        if (buf.length() != 0) {
            return buf.substring(1);
        }
        return "";
    }
}
