/*
 *     RefJavaElement.java  Jan 14, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.xref;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.j2xml.JavaXMLText;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.dom.*;
import org.w3c.dom.*;

public abstract class RefJavaElement extends JavaElement {
    protected String name = null;
    protected String fqn = null;
    protected String ref = null;

    protected RefJavaElement() {
    }

    public RefJavaElement(Node node) {
        super(node);
    }

    public String getName() {
        if (name == null) {
            name = getIdentName(elem);
        }
        return name;
    }

    public String getFqn() {
        if (fqn == null) {
            fqn = elem.getAttribute(JXConstants.FqnAttr);
        }
        return fqn;
    }

    public String getId() {
        return super.getId();
    }

    public String getNodeName() {
        return elem.getNodeName();
    }

    public String getSort() {
        return getElemSort(elem);
    }

    public String getFileName() {
        return getJavaFile(elem).getName();
    }

    public boolean isValid() {
        return getRefJavaClass() != null;
    }

    public String getRefFqn() {
        if (ref == null) {
            Element ident = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
            if (ident != null) {
                ref = ident.getAttribute(JXConstants.RefAttr);
            }
        }
        return ref;
    }

    public String getRefId() {
        if (getRefFqn().length() != 0) {
            Element ident = (Element)getFirstChildNode(elem, JXConstants.IdentElem);
            return getElemId(ident);
        }
        return null;
    }

    public String getRefName() {
        return getName();
    }

    public abstract JavaElement getRefJavaElem();

    public int getRefLine() {
        JavaElement jelem = getRefJavaElem();
        if (jelem != null) {
            return jelem.getLine();
        }
        return 0;
    }

    public JavaClass getRefJavaClass() {
        return Repository.getJavaClass(getRefFqn(), getJavaFile());
    }

    public String getRefFileName() {
        JavaClass jc = getRefJavaClass();
        if (jc != null) {
            return jc.getJavaFile().getName();
        }
        return "";
    }

    public JavaClass getJavaClass() {
        return getJavaClass(getElem());
    }
}
