/*
 *     ReferenceUtil.java  Jan 14, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.xref;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class ReferenceUtil {
    private static ArrayList id2je = new ArrayList();

    public static void makeId2JavaElement() {
        id2je.clear();

        Iterator it = Project.getAllFiles().iterator();
        while (it.hasNext()) {
            JavaFile jfile = (JavaFile)it.next();

            collectClassReference(jfile);
            collectFieldReference(jfile);
            collectMethodReference(jfile);
        }
    }     

    private static void collectClassReference(JavaFile jfile) {
        Iterator it = jfile.getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            RefJavaClass rjc = new RefJavaClass(elem);
            if (rjc.isObjectRef() || rjc.isClassRef()) {
                entryReference(rjc);
            }
        }
    }

    private static void collectFieldReference(JavaFile jfile) {
        Iterator it = jfile.getExpressionNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            RefJavaField rjf = new RefJavaField(elem);
            if (rjf.isFieldAccess()) {
                entryReference(rjf);
            }
        }
    }

    private static void collectMethodReference(JavaFile jfile) {
        Iterator it = jfile.getExpressionNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            RefJavaMethod rjm = new RefJavaMethod(elem);
            if (rjm.isMethodCall()) {
                entryReference(rjm);
            }
        }
    }

    private static void entryReference(RefJavaElement rje) {
        if (rje.getRefJavaClass() != null) {
            String id = rje.getRefFileName() + File.pathSeparator + rje.getRefId();

            if (GetXref.isInProject(rje.getRefFileName())) {
                id2je.add(new Id2JavaElement(id, rje));
            }
        }
    }

    public static ArrayList getReferringNodes(String id) {
        ArrayList nodes = new ArrayList();

        Iterator it = id2je.iterator();
            while (it.hasNext()) {
            Id2JavaElement e = (Id2JavaElement)it.next();
            if (e.getId().equals(id)) {
                nodes.add(e.getRefJavaElement());
            }
        }
        return nodes;
    }
}

class Id2JavaElement {
    private String id;
    private RefJavaElement rje;

    Id2JavaElement(String id, RefJavaElement rje) {
        this.id = id;
        this.rje = rje;
    }

    String getId() {
        return id;
    }

    RefJavaElement getRefJavaElement() {
        return rje;
    }
}
