/*
 *     RelationUtil.java  Dec 2, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.xref;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class RelationUtil {
    private static HashMap overridingMap = new HashMap();  // JavaMethod/JavaMethodList
    private static HashMap overriddenMap = new HashMap();  // JavaMethod/JavaMethodList
    private static HashMap overloadingMap = new HashMap();  // JavaMethod/JavaMethodList
    private static HashMap overloadedMap = new HashMap();  // JavaMethod/JavaMethodList
    private static HashMap implementingMap = new HashMap();  // JavaMethod/JavaMethodList
    private static HashMap implementedMap = new HashMap();  // JavaMethod/JavaMethodList

    public static void collectRelations() {
        overridingMap.clear();
        overriddenMap.clear();
        overloadingMap.clear();
        overloadedMap.clear();
        implementingMap.clear();
        implementedMap.clear();

        Iterator it = Project.getAllClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            collectRelations(jc);
        }

        it = Project.getAllInterfaces().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            collectRelations(jc);
        }
    }

    private static void collectRelations(JavaClass jc) {
        Iterator it = jc.getAllMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();
            collectRelations(jm);
        }
    }

    private static void collectRelations(JavaMethod jm) {
        collectRelations(jm, GenClassXML.getOverridingMethods(jm), overridingMap, overriddenMap);
        collectRelations(jm, GenClassXML.getOverloadingMethods(jm), overloadingMap, overloadedMap);
        collectRelations(jm, GenClassXML.getImplementingMethods(jm), implementingMap, implementedMap);
    }

    private static void collectRelations(JavaMethod jm, JavaMethodList targets, HashMap fmap, HashMap bmap) {
        JavaMethodList fmethods = new JavaMethodList();
        String fkey = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        fmap.put(fkey, fmethods);

        Iterator it = targets.iterator();
        while (it.hasNext()) {
            JavaMethod m = (JavaMethod)it.next();
            fmethods.add(m);
            
            String bkey = m.getJavaClass().getFqn() + "#" + m.getSignature();
            JavaMethodList bmethods = (JavaMethodList)bmap.get(bkey);
            if (bmethods == null) {
                bmethods = new JavaMethodList();
                bmap.put(bkey, bmethods);
            }
            bmethods.add(jm);
        }
    }

    public static JavaMethodList getOverridingMethods(JavaMethod jm) {
        String key = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        JavaMethodList methods = (JavaMethodList)overridingMap.get(key);
        if (methods != null) {
            return methods;
        }
        return new JavaMethodList();
    }

    public static JavaMethodList getOverriddenMethods(JavaMethod jm) {
        String key = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        JavaMethodList methods = (JavaMethodList)overriddenMap.get(key);
        if (methods != null) {
            return methods;
        }
        return new JavaMethodList();
    }

    public static JavaMethodList getOverloadingMethods(JavaMethod jm) {
        String key = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        JavaMethodList methods = (JavaMethodList)overloadingMap.get(key);
        if (methods != null) {
            return methods;
        }
        return new JavaMethodList();
    }

    public static JavaMethodList getOverloadedMethods(JavaMethod jm) {
        String key = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        JavaMethodList methods = (JavaMethodList)overloadedMap.get(key);
        if (methods != null) {
            return methods;
        }
        return new JavaMethodList();
    }

    public static JavaMethodList getImplementingMethods(JavaMethod jm) {
        String key = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        JavaMethodList methods = (JavaMethodList)implementingMap.get(key);
        if (methods != null) {
            return methods;
        }
        return new JavaMethodList();
    }

    public static JavaMethodList getImplementedMethods(JavaMethod jm) {
        String key = jm.getJavaClass().getFqn() + "#" + jm.getSignature();
        JavaMethodList methods = (JavaMethodList)implementedMap.get(key);
        if (methods != null) {
            return methods;
        }
        return new JavaMethodList();
    }
}
